<% @title = "jquery-ujs test" %>

<%= test 'data-confirm', 'data-remote', 'data-disable', 'data-disable-with', 'call-remote', 'call-remote-callbacks', 'data-method', 'override', 'csrf-refresh', 'csrf-token' %>

<h1 id="qunit-header"><%= @title %></h1>
<div id="jquery-cdn">
  CDN:
  <%= cdn_link 'jquery' %> •
  <%= cdn_link 'googleapis' %>
</div>
<div id="jquery-version">
  jQuery version:

  <% jquery_versions.each do |v| %>
    <%= ' • ' if v != jquery_versions.first %>
    <%= jquery_link v %>
  <% end %>
 <%= (' • ' + jquery_link('edge')) if File.exist?(settings.root + '/public/vendor/jquery.js') %>
</div>
<h2 id="qunit-banner"></h2>
<div id="qunit-testrunner-toolbar"></div>
<h2 id="qunit-userAgent"></h2>
<ol id="qunit-tests"></ol>

<div id="qunit-fixture"></div>
