<!DOCTYPE html>
<html id="html">
  <head>
    <title><%= @title %></title>
    <link href="/vendor/qunit.css" media="screen" rel="stylesheet" type="text/css" media="screen, projection" />
    <style>
      #jquery-cdn, #jquery-version {
        padding: 0 2em .8em 0;
        text-align: right;
        font-family: sans-serif;
        line-height: 1;
        color: #8699A4;
        background-color: #0d3349;
      }

      #jquery-cdn a, #jquery-version a {
        color: white;
        text-decoration: underline;
      }
    </style>

    <%= script_tag jquery_src %>
    <script>
      // This is for test in override.js.
      // Must go after jQuery is loaded, but before jquery-ujs.
      $(document).on('rails:attachBindings', function() {
        $.rails.linkClickSelector += ', a[data-custom-remote-link]';
      });
    </script>
    <%= script_tag "/src/rails.js" %>
  </head>

  <body id="body">
    <%= yield %>
  </body>
</html>
